
/* 
   Project:     SOFTSUSY 
   File:        physpars.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html

   $Log: physpars.cpp,v $
   Revision 1.3  2002/04/15 09:09:04  allanach
   *** empty log message ***

   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info
*/

#include "physpars.h"

const sPhysical & sPhysical::operator=(const sPhysical &s) {
  if (this == &s) return *this;
  mhiggs = s.mhiggs; msnu = s.msnu; 
  mch = s.mch; mneut = s.mneut; mixNeut = s.mixNeut;
  thetaL = s.thetaL; thetaR = s.thetaR; mGluino = s.mGluino;
  thetat = s.thetat; thetab = s.thetab; thetatau = s.thetatau;
  mu = s.mu; md = s.md; me = s.me; thetaH = s.thetaH;
  t1OV1Ms = s.t1OV1Ms; t2OV2Ms = s.t2OV2Ms;
  aChi0ChicW = s.aChi0ChicW; bChi0ChicW = s.bChi0ChicW;
  return *this;
}

// a should be in C convention ie start from index zero
void sPhysical::display(double *a) const {
  a[0] = mhiggs.display(1); a[1] = mhiggs.display(2); 
  a[2] = mhiggs.display(3); a[3] = mhiggs.display(4);

  a[4] = msnu.display(1); a[5] = msnu.display(2); a[6] = msnu.display(3);

  a[7] = mch.display(1); a[8] = mch.display(2);

  a[9] = mneut.display(1); a[10] = mneut.display(2); 
  a[11] = mneut.display(3); a[12] = mneut.display(4);

  a[13] = mGluino;

  int i, j, k = 13; 
  for (i=1; i<=4; i++)
    for (j=1; j<=4; j++) {
      k++;
      a[k] = mixNeut.display(i, j);  
    }

  a[30] = thetaL; a[31] = thetaR; 
  a[32] = thetat; a[33] = thetab; a[34] = thetatau;

  k = 34;
  for (i=1; i<=2; i++)
    for (j=1; j<=3; j++) {
      k++;
      a[k] = mu.display(i, j);
      a[k+6] = md.display(i, j);
      a[k+12] = me.display(i, j);
    }  

  a[53] = thetaH;
}

#define HR "---------------------------------------------------------------\n"

ostream & operator <<(ostream & left, const sPhysical &s) {
  left << HR;
  left << "Physical MSSM parameters";
  left << "\nmh^0: " << s.mhiggs.display(1) << " mA^0: " << s.mhiggs.display(2)
       << " mH^0: " << 
    s.mhiggs.display(3) << " mH^+-: " << s.mhiggs.display(4) << "\n";
  left << "alpha: " << s.thetaH << "\n";
  left << "sneutrinos" << s.msnu; 
  left << "mU~" << s.mu << "mD~" << s.md << "mE~" << s.me;
  left << "thetat: " << s.thetat << " thetab: " << s.thetab << 
    " thetatau: " << s.thetatau << "\n";
  left << "mGluino:  " << s.mGluino << "\n";
  left << "charginos" << s.mch;
  left << "thetaL: " << s.thetaL << " thetaR: " << s.thetaR << "\n";
  left << "neutralinos" << s.mneut;
  left << "neutralino mixing matrix " << s.mixNeut;
  return left;
}

istream & operator >>(istream & left, sPhysical &s) {
  char c[70];
  left >> c >> c >> c >> c;
  DoubleVector mh(4);
  left >> c >> s.mhiggs(1) >> c >> s.mhiggs(2)
       >> c >> s.mhiggs(3) >> c >> s.mhiggs(4);
  left >> c >> s.thetaH;
  left >> s.msnu; 
  left >> c >> s.mu >> c >> s.md >> c >> s.me;
  left >> c >> s.thetat >> c >> s.thetab >> 
    c >> s.thetatau;
  left >> c >> s.mGluino;
  left >> s.mch;
  left >> c >> s.thetaL >> c >> s.thetaR;
  left >> s.mneut;
  left >> c >> c >> c >> c >> s.mixNeut;
  return left;
}

#undef HR

ostream & operator <<(ostream &st, const sProblem & p) {
  if (!p.test()) return st;
  st << "[ ";
  if (p.noMuConvergence) st << "No mu convergence ";
  if (p.noRhoConvergence) st << "No rho convergence ";
  if (p.nonperturbative) st << "Non-perturbative ";
  if (p.noConvergence) st << "No convergence ";
  if (p.tachyon) st << "Tachyon ";
  if (p.muSqWrongSign) st << "MuSqWrongsign ";
  if (p.b) st << "B-problem ";
  if (p.higgsUfb) st << "Higgs potential ufb ";
  st << "]";
  return st;
};

const sProblem & sProblem::operator=(const sProblem &s) {
  if (this == &s) return *this;
  noMuConvergence = s.noMuConvergence;
  noRhoConvergence = s.noRhoConvergence;
  nonperturbative = s.nonperturbative;
  noConvergence = s.noConvergence;
  tachyon = s.tachyon;
  muSqWrongSign = s.muSqWrongSign;
  higgsUfb = s.higgsUfb;
  b = s.b;
  return *this;
}

