
/* 
   Project:     SOFTSUSY 
   File:        rgeroutines.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html

   $Log: rgeroutines.cpp,v $
   Revision 1.3  2002/04/15 09:13:05  allanach
   *** empty log message ***

   Revision 1.2  2002/04/11 12:37:55  allanach
   *** empty log message ***

   Revision 1.8  2001/11/30 10:16:04  allanach
   Scan in model lines altered

   Revision 1.7  2001/10/23 12:53:58  allanach
   Easier user interface utilised

   Revision 1.6  2001/10/22 09:19:28  allanach
   Fixed missing fermion contribution in piZZT. Turned some integers to
   decimals for performance on more platforms.

   Revision 1.5  2001/10/04 19:26:34  allanach
   New version deals with AMSB correctly

   Revision 1.4  2001/10/03 13:08:55  allanach

   Revision 1.3  2001/09/28 14:53:32  allanach
   Model line scan function m12Scan() added. GMSB and AMSB boundary conditions
   added.

   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info
*/

#include <rgeroutines.h>

//Produces a plot of parameters verses renormalisation scale in mSUGRA
void scaledParsSugra() {
  QedQcd oneset;
  readIn(oneset, "massIn");
  oneset.toMz();

  double mgut = 2.0e16;
  int sgnMu = -1;
  double m0 = 200.0, m12 = 100.0, a0 = 0.0, tanb = 2.0;
  cout << "#m0=" << m0 << " a0=" << a0 << " m12=" << m12 << " tanb=" << tanb
       << " signmu=" << sgnMu << endl;
  DoubleVector pars(3);
  translateSugra(pars, m0, m12, a0);

  MssmSoftsusy r;
  r.lowOrg(sugraBcs, mgut, pars, sgnMu, tanb, oneset);  
  MssmSoftsusy s(r);

  double startlnMu = log(MZ), endlnMu = log(1.0e3);
  int numPoints = 50;
  int c; for (c=0; c<=numPoints; c++) {
    double lnmu = startlnMu + double(c) * (endlnMu - startlnMu) / numPoints;
    s.runto(exp(lnmu));
    cout << exp(lnmu) << " " 
	 << (s.displayMh2Squared() + sqr(s.displaySusyMu()))
      * (s.displayMh1Squared() + sqr(s.displaySusyMu())) -
      sqr(s.displayM3Squared())
	 << endl;
  }
  
}

// for a given string scale and sign of mu parameter, produces a 50x50 grid of
// fine tuning in m12 vs M0 plane
// accuracy: 0 no SUSY thresholds except for MSusy
// 1: SUSY particle thresholds
// 2: threshold corrections to SUSY parameters
void contUniversalM0m12(double tanb, double a0, double mgut, int sgnMu, int
			   accuracy) {
  cout << "%MGUT=" << mgut << " sgnMu=" << sgnMu << " tanb=" <<
    tanb << " a0=" << a0 << " accuracy=" << EPS << "\n";
  
  double startm12 = 100.0, endm12 = 1000.0;
  double startm0 = 100.0, endm0 = 4000.0;
  int numPoints = 15; 
      
  int count, count2;

  cout << "%     m0     " << "     m12    ";
  printShortInitialise();
  cout << "   cMu0   " << "    cB0   " << "   cM0    " << "   cm12   "
       << "    cA0   " << endl;

  for (count = 0; count <= numPoints; count++) 
    for (count2 = 0; count2 <= numPoints; count2++) {
      double m12 = startm12 + count * (endm12 - startm12) / numPoints;
      double m0 = startm0 + count2 * (endm0 - startm0) / numPoints;

      QedQcd oneset;
      readIn(oneset, "massIn");

      oneset.toMz();
      
      DoubleVector pars(3); 
      
      MssmSoftsusy r;   
        
      translateSugra(pars, m0, m12, a0);
      r.lowOrg(sugraBcs, mgut, pars, sgnMu, tanb, oneset);
      cout << m0 << " " << m12;
      r.printShort(); 

      DoubleVector ft(6);
      ft = r.fineTune(sugraBcs, pars, mgut);
      cout << ft(1) << " " << ft(2) << " " << ft(3) << " " << ft(4) 
	   << " " << ft(5) << " " << ft(6);
      if (r.displayProblem().test()) cout << "%" << r.displayProblem(); 
      cout << endl;      

    }
}

// for a given string scale and sign of mu parameter, produces a 50x50 grid of
// soft mass in m32 sintheta vs tan beta plane
// accuracy: 0 no SUSY thresholds except for MSusy
// 1: SUSY particle thresholds
// 2: threshold corrections to SUSY parameters
// susyBreaking: 
// 1: dilaton domination
// 2: moduli domination
void contDomination(double mgut, int sgnMu, int accuracy) {
  // Grid of (numPoints+1)x(numPoints+1)
  int numPoints = 15;

  double startM32 = 50.0, endM32 = 350.0;
  double startTanb = 2.0, endTanb = 47.0;
      
  int count, count2;
  cout << "%    m32     " << "     tanb   ";  
  printShortInitialise();
  cout << "    cMu0   " << "     cB0  " << "     cM32  " 
       << "    cHT    " << endl;

  DoubleVector ft(4);
  MssmSoftsusy r;

  for (count2 = 0; count2 <= numPoints; count2++)
    for (count = 0; count <= numPoints; count++) {
	double m32 = startM32 + count * (endM32 - startM32) / numPoints;
	double tanb = startTanb + count2 * (endTanb - startTanb) /
	  numPoints;
	double m0 = m32, m12 = sqrt(3.0) * m32, a0 = -m12;

	r = doPointUniversalM0m12(m0, m12, tanb, a0, mgut, sgnMu,
				      accuracy, ft);

	cout << m32 << " " << tanb << " ";
	r.printShort(); cout << ft;
	}

  cout.flush(); return;
}

inline void translateSugra(DoubleVector & pars, double m0, double m12,
			    double a0) {
  pars(1) = m0; pars(2) = m12; pars(3) = a0;
}

MssmSoftsusy pointDomination(double tanb, double
			       mgut, int sgnMu, int accuracy, DoubleVector &
			       ft, double m32,
			       double epsilon) {
  QedQcd oneset;
  readIn(oneset, "massIn");
  oneset.toMz();
  
  double m0, m12, a0;

  DoubleVector pars(3); 
  
  m0 = m32; m12 = sqrt(3.0) * m32; a0 = -m12;
  translateSugra(pars, m0, m12, a0);
  ft.setEnd(4);
  MssmSoftsusy r;
  r.lowOrg(sugraBcs, mgut, pars, sgnMu,  tanb, oneset); 
  
  return r;
}

void nonUniGauginos(MssmSoftsusy & m, const DoubleVector & inputParameters)
{
  double m0 = inputParameters.display(1);
  double m12 = inputParameters.display(2);
  double a0 = inputParameters.display(3);

  // Sets scalar soft masses equal to m0, fermion ones to m12 and sets the
  // trilinear scalar coupling to be a0
  m.standardSugra(m0, m12, a0);
  m.setGauginoMass(1, 1.6 * m12);
    
  return;
}

static int n5 = 0; // number of 5+5bar's, must be set.

void gmsbBcs(MssmSoftsusy & m, const DoubleVector & inputParameters)
{
  if (n5 == 0) {
    cout << "n5 not set in rgeroutines.cpp:gmsbBcs\n";
    exit(1);
  }

  double mMess = inputParameters.display(1);
  double lambda = inputParameters.display(2);

  // Sets scalar soft masses equal to m0, fermion ones to m12 and sets the
  // trilinear scalar coupling to be a0
  m.minimalGmsb(n5, lambda, mMess);
    
  return;
}

void amsbBcs(MssmSoftsusy & m, const DoubleVector & inputParameters)
{
  double m32 = inputParameters.display(1);
  double m0 = inputParameters.display(2);

  m.standardSugra(m0, 0., 0.);
  m.addAmsb(m32);

  return;
}


// Scans through m12 given other SUGRA parameters, used for making 1d plots.
void m12Scan() {
  
  QedQcd oneset;
  readIn(oneset, "massIn");
  oneset.toMz();

  // Default parameters
  int sgnMu = 1;
  double m0 = 400., M12 = 400., a0 = 0.0, tanb = 10., mgut = 1.9e16;

  double endM12 = 700., startM12 = 100.;
  //  double endLambda = 150.0e3, startLambda = 30.0e3;
  int i, numPts = 50; 
  
  DoubleVector pars(3);

  void (*bCType)(MssmSoftsusy & m, const DoubleVector & inputParameters);
  
  int option;
  /*
  for (option = 1; option <= 8; option++) {

    cout << "Option: " << option << endl;
    
    for (i=0; i<=numPts; i++)
      {
	
	MssmSoftsusy r;
	
	switch(option) {
	case 1:
	  // Model line A
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 0.4 * M12;
	  a0 = -0.4 * M12;
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
	case 2:
	  // Model line B
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 0.5 * M12;
	  a0 = 0.0;
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &nonUniGauginos;
	  break;
	case 3:
	  // Model line C
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = M12;
	  a0 = 0.;
	  tanb = 35.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
	case 4:
	  // Model line D
	  { 
	    double lambda = (endLambda - startLambda) / double(numPts) 
	      * double(i) + startLambda; 
	    double mMess = 2.0 * lambda;
	    tanb = 15.; n5 = 3;
	    pars.setEnd(2);
	    pars(1) = mMess; pars(2) = lambda;
	    mgut = mMess;
	    cout << lambda;
	  }
	  bCType = &gmsbBcs;
	  break;
	case 5:
	  // Model line E
	  { 
	    endLambda = 250.0e3;
	    double lambda = (endLambda - startLambda) / double(numPts) 
	      * double(i) + startLambda; 
	    double mMess = 2.0 * lambda;
	    tanb = 15.; n5 = 1;
	    pars.setEnd(2);
	    pars(1) = mMess; pars(2) = lambda;
	    mgut = mMess;
	    cout << lambda;
	  }
	  bCType = &gmsbBcs;
	  break;
	case 6:
	  // Model line F
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 2.0 * M12 + 800.;
	  a0 = 0.;
	  pars.setEnd(3);
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
	case 7:
	  // Model line G
	  {
	    double endM32 = 150., startM32 = 400.0;
	    double M32 = 
	      (endM32 - startM32) / double(numPts) * double(i) + startM32;

	    m0 = M32;
	    M32 = 50.0e3;
	    pars.setEnd(2);
	    pars(1) = M32; pars(2) = m0;
	    cout << m0;
	  }
	  tanb = 10.;
	  bCType = &amsbBcs;
	  break;
	case 8:
	  // Model line F
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 0.25 * M12 - 9.0;
	  a0 = 0.;
	  pars.setEnd(3);
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
      default:
	cerr << "m12scan called with illegal " << option << "\n";
	exit(1); 
	break;
	}
      
      r.lowOrg(bCType, mgut, pars, sgnMu, tanb, oneset);

      if (option == 1 || option == 2 || option == 3 || option == 6) 
	cout << M12; 

      r.printShort();   

      //      r.runto(r.calcMs());
      //cout << r; exit(1); //DEBUG

      if (r.displayProblem().test()) cout << "%" << r.displayProblem(); 
      cout << endl;      
      }

  cout << endl << endl;
*/

  for (option = 1; option <= 8; option++) {
    for (i=0; i<=numPts; i++) {
	
	MssmSoftsusy r;
	
	switch(option) {
	case 1:
	  // Model line A
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 0.4 * M12;
	  a0 = -0.4 * M12;
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
	case 2:
	  // Model line B
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 0.5 * M12;
	  a0 = 0.0;
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &nonUniGauginos;
	  break;
	case 6:
	  // Model line F
	  M12 = (endM12 - startM12) / double(numPts) * double(i) + startM12;
	  m0 = 2.0 * M12 + 800.;
	  a0 = 0.;
	  pars.setEnd(3);
	  tanb = 10.;
	  translateSugra(pars, m0, M12, a0);
	  bCType = &sugraBcs;
	  break;
      }
      
  	if (option==1 || option ==2 || option ==6){
	  r.lowOrg(bCType, mgut, pars, sgnMu, tanb, oneset);
	  
	  if (option == 1 || option == 2 || option == 3 || option == 6) 
	    cout << M12; 
	  
	  r.printShort(); 

	  if (r.displayProblem().test()) cout << "%" << r.displayProblem(); 
	  cout << endl;      
	} // option if
    }  // numpts loop
  

      cout << endl << endl;
  }  // option loop
}

// Runs one point in SUGRA space, returning Fine Tuning vector in ft
MssmSoftsusy doPointUniversalM0m12(double m0, double m12, double tanb,
					double a0, double mgut, int sgnMu, 
					int accuracy, DoubleVector & ft) {
  QedQcd oneset;
  readIn(oneset, "massIn");

  oneset.toMz();

  DoubleVector pars(3); translateSugra(pars, m0, m12, a0);

  MssmSoftsusy r;
  r.lowOrg(sugraBcs, mgut, pars, sgnMu, tanb, oneset);
  ft = r.fineTune(sugraBcs, pars, mgut);  

  return r;
}

// writes one universal sugra point to standard output
void pointUniversalM0m12(double m0, double m12, double tanb, double a0,
			    double mgut, int sgnMu, int accuracy) {
  DoubleVector ft(6);

  // none=sugra/dil for FT calculation
  MssmSoftsusy r(doPointUniversalM0m12(m0,  m12,  tanb, a0,  mgut,
					    sgnMu, accuracy, ft));

  cout << m0 << " " << m12; cout.flush();  
  r.printShort(); cout << ft;
}




