
/*
   Project:     SOFTSUSY 
   File:        utils.h
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: A few handy bits and pieces - little mathematical functions 
                and the like

   $Log: utils.h,v $
   Revision 1.3  2002/05/01 16:02:20  allanach
   Added checkTolerance subroutine

*/

#ifndef UTILS_H
#define UTILS_H

#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <def.h>

int theta(double);
void check(double a, double b, char *);
void outputCharacteristics(int);
inline double sqr(double a) { return a * a; }
inline double maximum(double a, double b) { return ((a > b) ? a : b); }
inline double minimum(double a, double b) { return ((a < b) ? a : b); }
inline int minimum(int a, int b) { return ((a < b) ? a : b); }
double toleranceCheck(double sTin, double sTout);

// checks if ABSOLUTE (or squared) values are closer than tol
inline bool close(double m1, double m2, double tol){
  return ( fabs(1.0 - 
		fabs(minimum(fabs(m1), fabs(m2)) / maximum(fabs(m1), fabs(m2)))) <
	    tol );
};

// Returns |a| with sign of b in front
inline double sign(double a, double b) 
{ return ((b) >= 0.0 ? fabs(a) : -fabs(a)); };

inline int sgn(double x)
{ return (x >= 0.0 ? 1 : -1); };

#endif
